@@ -1,6 +1,12 @@
+/*
+For general Scribus (>=1.3.2) copyright and licensing information please refer
+to the COPYING file provided with the program. Following this notice may exist
+a copyright and/or license notice that predates the release of Scribus 1.3.2
+for which a new license (GPL+exception) is in place.
+*/
 /***************************************************************************
  *   Copyright (C) 2004 by Riku Leino                                      *
- *   riku.leino@gmail.com                                                      *
+ *   tsoots@gmail.com                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -35,6 +41,7 @@ const QString gtFont::fontWeights[FontWeightMAX] =
 	"Demi",
 	"Heavy",
 	"Light",
+	"Lite",
 	"Medium",
 	"Regular",
 	"Roman"
@@ -62,6 +69,7 @@ const QString gtFont::fontWidths[FontWidthMAX] =
 
 gtFont::gtFont()
 {
+	setflags = 0;
 	noEffects();
 	name        = "";
 	family      = "";
@@ -69,13 +77,13 @@ gtFont::gtFont()
 	slant       = "";
 	width       = "";
 	append      = "";
-	size        = 12;
+	size        = 120;
 	color       = "Black";
 	shade       = 100;
 	strokeColor = "Black";
 	strokeShade = 100;
-	hscale      = 100;
-	kerning     = 0.0;
+	hscale      = 1000;
+	kerning     = 0;
 	useFullName = true;
 	weightIndex = 0;
 	slantIndex  = 1;
@@ -107,6 +115,12 @@ gtFont::gtFont(const gtFont& f)
 	weightIndex = f.weightIndex;
 	slantIndex  = f.slantIndex;
 	widthIndex  = f.widthIndex;
+	smallestIndex  = f.smallestIndex;
+	biggestIndex   = f.biggestIndex;
+	index          = f.index;
+	tmpWeightIndex = f.tmpWeightIndex;
+	tmpSlantIndex  = f.tmpSlantIndex;
+	tmpWidthIndex  = f.tmpWidthIndex;
 	fontEffects[NORMAL]        = f.fontEffects[NORMAL];
 	fontEffects[UNDERLINE]     = f.fontEffects[UNDERLINE];
 	fontEffects[STRIKETHROUGH] = f.fontEffects[STRIKETHROUGH];
@@ -114,6 +128,12 @@ gtFont::gtFont(const gtFont& f)
 	fontEffects[SUPERSCRIPT]   = f.fontEffects[SUPERSCRIPT];
 	fontEffects[SUBSCRIPT]     = f.fontEffects[SUBSCRIPT];
 	fontEffects[OUTLINE]       = f.fontEffects[OUTLINE];
+	setflags = f.setflags;
+}
+
+int gtFont::getFlags()
+{
+	return setflags;
 }
 
 bool gtFont::isToggled(FontEffect fe)
@@ -150,6 +170,7 @@ bool gtFont::toggleEffect(FontEffect fe)
 			if (fontEffects[fe])
 				fontEffects[NORMAL] = false;
 	}
+	setflags |= effectWasSet;
 	return fontEffects[fe];
 }
 
@@ -181,12 +202,14 @@ void gtFont::setName(QString newName)
 	setWidth(NO_WIDTH);
 	parseName();
 	useFullName = true;
+	setflags |= familyWasSet;
 }
 
 void gtFont::setFamily(QString newFamily)
 {
 	family = newFamily;
 	useFullName = false;
+	setflags |= familyWasSet;
 }
 
 QString gtFont::getFamily()
@@ -198,10 +221,12 @@ void gtFont::setWeight(FontWeight newWeight)
 {
 	weight = fontWeights[newWeight];
 	useFullName = false;
+	setflags |= weightWasSet;
 	if ((newWeight == ROMAN) || (newWeight == REGULAR))
 	{
 		setSlant(NO_SLANT);
 		setWidth(NO_WIDTH);
+		setflags &= ~weightWasSet;
 	}
 	if (weightIndex < 0)
 	{
@@ -215,11 +240,13 @@ void gtFont::setWeight(QString newWeight)
 {
 	weight = newWeight;
 	useFullName = false;
-		if ((newWeight == fontWeights[ROMAN]) || 
-		    (newWeight == fontWeights[REGULAR]))
+	setflags |= weightWasSet;
+	if ((newWeight == fontWeights[ROMAN]) || 
+		(newWeight == fontWeights[REGULAR]))
 	{
 		setSlant(NO_SLANT);
 		setWidth(NO_WIDTH);
+		setflags &= ~weightWasSet;
 	}
 	if (weightIndex < 0)
 	{
@@ -238,12 +265,14 @@ void gtFont::setSlant(FontSlant newSlant)
 {
 	slant = fontSlants[newSlant];
 	useFullName = false;
+	setflags &= ~slantWasSet;
 	if (newSlant != NO_SLANT)
 	{
 		if (weight == fontWeights[REGULAR])
 			setWeight(NO_WEIGHT);
 		else if (weight == fontWeights[ROMAN])
 			setWeight(NO_WEIGHT);
+		setflags |= slantWasSet;
 	}
 	if (slantIndex < 0)
 	{
@@ -257,12 +286,14 @@ void gtFont::setSlant(QString newSlant)
 {
 	slant = newSlant;
 	useFullName = false;
-	if (newSlant != "")
+	setflags &= ~slantWasSet;
+	if (!newSlant.isEmpty())
 	{
 		if (weight == fontWeights[REGULAR])
 			setWeight(NO_WEIGHT);
 		else if (weight == fontWeights[ROMAN])
 			setWeight(NO_WEIGHT);
+		setflags |= slantWasSet;
 	}
 	if (slantIndex < 0)
 	{
@@ -281,12 +312,14 @@ void gtFont::setWidth(FontWidth newWidth)
 {
 	width = fontWidths[newWidth];
 	useFullName = false;
+	setflags &= ~widthWasSet;
 	if (newWidth != NO_WIDTH)
 	{
 		if (weight == fontWeights[REGULAR])
 			setWeight(NO_WEIGHT);
 		else if (weight == fontWeights[ROMAN])
 			setWeight(NO_WEIGHT);
+		setflags |= widthWasSet;
 	}
 	if (widthIndex < 0)
 	{
@@ -300,12 +333,14 @@ void gtFont::setWidth(QString newWidth)
 {
 	width = newWidth;
 	useFullName = false;
-	if (newWidth != "")
+	setflags &= ~widthWasSet;
+	if (!newWidth.isEmpty())
 	{
 		if (weight == fontWeights[REGULAR])
 			setWeight(NO_WEIGHT);
 		else if (weight == fontWeights[ROMAN])
 			setWeight(NO_WEIGHT);
+		setflags |= widthWasSet;
 	}
 	if (widthIndex < 0)
 	{
@@ -323,32 +358,37 @@ QString gtFont::getWidth()
 void gtFont::setSize(int newSize)
 {
 	size = newSize;
+	setflags |= sizeWasSet;
 }
 
 void gtFont::setSize(double newSize)
 {
-	double tmp = newSize * 10;
-	size = static_cast<int>(tmp);
+	size = static_cast<int>(newSize);
+	setflags |= sizeWasSet;
 }
 
 void gtFont::setColor(QString newColor)
 {
 	color = newColor;
+	setflags |= fillColorWasSet;
 }
 
 void gtFont::setShade(int newShade)
 {
 	shade = newShade;
+	setflags |= fillShadeWasSet;
 }
 
 void gtFont::setStrokeColor(QString newColor)
 {
 	strokeColor = newColor;
+	setflags |= strokeColorWasSet;
 }
 
 void gtFont::setStrokeShade(int newShade)
 {
 	strokeShade = newShade;
+	setflags |= strokeShadeWasSet;
 }
 
 QString gtFont::getName()
@@ -357,26 +397,30 @@ QString gtFont::getName()
 		return name;
 
 	QString name2 = family + " ";
+
 	if (weightIndex == 0)
 		name2 += weight + " ";
 	else if (slantIndex == 0) 
 		name2 += slant + " ";
 	else if (widthIndex == 0)
 		name2 += width + " ";
+
 	if (weightIndex == 1)
 		name2 += weight + " ";
 	else if (slantIndex == 1) 
 		name2 += slant + " ";
 	else if (widthIndex == 1)
 		name2 += width + " ";
+
 	if (weightIndex == 2)
 		name2 += weight + " ";
 	else if (slantIndex == 2) 
 		name2 += slant + " ";
 	else if (widthIndex == 2)
 		name2 += width + " ";
+
 	name2 += append;
-	name2 = name2.simplifyWhiteSpace();
+	name2 = name2.simplified();
 	return name2;
 }
 
@@ -422,15 +466,15 @@ QString gtFont::getName(uint i)
 			fname = fname + " " + append + " " + width + " " + slant + " " + weight;
 			break;
 		case 12:
-			if ((append == "") && (weight == "") && (slant == "") && (width == ""))
+			if ((append.isEmpty()) && (weight.isEmpty()) && (slant.isEmpty()) && (width.isEmpty()))
 				fname = fname + " " + fontWeights[REGULAR];
 			break;
 		case 13:
-			if ((append == "") && (weight == "") && (slant == "") && (width == ""))
+			if ((append.isEmpty()) && (weight.isEmpty()) && (slant.isEmpty()) && (width.isEmpty()))
 				fname = fname + " " + fontWeights[ROMAN];
 			break;
 	}
-	fname = fname.simplifyWhiteSpace();
+	fname = fname.simplified();
 	return fname;
 }
 
@@ -468,6 +512,7 @@ void gtFont::noEffects()
 	fontEffects[SUPERSCRIPT]   = false;
 	fontEffects[SUBSCRIPT]     = false;
 	fontEffects[OUTLINE]       = false;
+	setflags &= ~effectWasSet;
 }
 
 int gtFont::getHscale()
@@ -475,19 +520,21 @@ int gtFont::getHscale()
 	return hscale;
 }
 
-void    gtFont::setHscale(int newHscale)
+void gtFont::setHscale(int newHscale)
 {
 	hscale = newHscale;
+	setflags |= hscaleWasSet;
 }
 
-double gtFont::getKerning()
+int gtFont::getKerning()
 {
 	return kerning;
 }
 
-void gtFont::setKerning(double newKerning)
+void gtFont::setKerning(int newKerning)
 {
 	kerning = newKerning;
+	setflags |= kerningWasSet;
 }
 
 void gtFont::parseName()
@@ -502,8 +549,6 @@ void gtFont::parseName()
 	parseSlant();
 	parseWidth();
 	parseFamily();
-
-
 }
 
 void gtFont::parseWeight()
@@ -511,18 +556,18 @@ void gtFont::parseWeight()
 	bool found = false;
 	for (int i = 1; i < FontWeightMAX; ++i)
 	{
-		index = find(name, fontWeights[i]); // f.e. Demi Bold
+		index = name.indexOf(fontWeights[i]); // f.e. Demi Bold
 		QString tmpWeight = "";
-		if ((index == -1) && (fontWeights[i].find(" ") != -1) && (fontWeights[i].find(" ") != 1))
+		if ((index == -1) && (fontWeights[i].indexOf(" ") != -1) && (fontWeights[i].indexOf(" ") != 1))
 		{
 			QString fw2 = fontWeights[i];
 			fw2.replace(" ", "-"); // f.e. Demi-Bold
-			index = find(name, fw2);
+			index = name.indexOf(fw2);
 			if (index == -1)
 			{
 				fw2 = fontWeights[i];
 				fw2.replace(" ", ""); // f.e. DemiBold
-				index = find(name, fw2);
+				index = name.indexOf(fw2);
 				if (index == -1)
 				{
 					fw2 = fontWeights[i];
@@ -531,7 +576,7 @@ void gtFont::parseWeight()
 					fw2.replace(" H", " h");
 					fw2.replace(" L", " l");
 					fw2.replace(" ", "");
-					index = find(name, fw2);
+					index = name.indexOf(fw2);
 					if (index != -1)
 						tmpWeight = fw2;
 				}
@@ -566,7 +611,7 @@ void gtFont::parseSlant()
 	bool found = false;
 	for (int i = 1; i < FontSlantMAX; ++i)
 	{
-		index = find(name, fontSlants[i]);
+		index = name.indexOf(fontSlants[i]);
 		if (index != -1)
 		{
 			slant = fontSlants[i];
@@ -588,18 +633,18 @@ void gtFont::parseWidth()
 	bool found = false;
 	for (int i = 1; i < FontWidthMAX; ++i)
 	{
-		index = find(name, fontWidths[i]);
+		index = name.indexOf(fontWidths[i]);
 		QString tmpWidth = "";
-		if ((index == -1) && (fontWidths[i].find(" ") != -1) && (fontWidths[i].find(" ") != 1))
+		if ((index == -1) && (fontWidths[i].indexOf(" ") != -1) && (fontWidths[i].indexOf(" ") != 1))
 		{
 			QString fw2 = fontWidths[i];
 			fw2.replace(" ", "-");
-			index = find(name, fw2);
+			index = name.indexOf(fw2);
 			if (index == -1)
 			{
 				fw2 = fontWidths[i];
 				fw2.replace(" ", "");
-				index = find(name, fw2);
+				index = name.indexOf(fw2);
 				if (index == -1)
 				{
 					fw2 = fontWidths[i];
@@ -608,7 +653,7 @@ void gtFont::parseWidth()
 					fw2.replace(" H", " h");
 					fw2.replace(" L", " l");
 					fw2.replace(" ", "");
-					index = find(name, fw2);
+					index = name.indexOf(fw2);
 					if (index != -1)
 						tmpWidth = fw2;
 				}
@@ -687,21 +732,21 @@ void gtFont::parseFamily()
 		family = name;
 	else
 		family = name.left(smallestIndex);
-	if (biggestIndex == -1)
+	if (biggestIndex == -1 || biggestIndex >= name.length())
 		append = "";
 	else
 		append = name.right(name.length() - biggestIndex - 1);
-	family = family.stripWhiteSpace();
+	family = family.trimmed();
 }
 
 int gtFont::find(const QString& where, const QString& what)
 {
 	QString realWhat = " " + what;
-	int index = where.findRev(realWhat); // f.e. Demi Bold
+	int index = where.lastIndexOf(realWhat); // f.e. Demi Bold
 	if (index != -1)
 	{
 		if (index + realWhat.length() != where.length())
-			if (where[index + realWhat.length() + 1] != " ")
+			if (where[index + realWhat.length() + 1] != ' ')
 				index = -1;
 	}
 	return index;
