@@ -4,19 +4,25 @@ to the COPYING file provided with the program. Following this notice may exist
 a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
-/* $Id: colorwheelwidget.cpp 4979 2006-03-23 10:12:18Z subik $ */
+
 #include "colorwheelwidget.h"
-#include "colorwheelwidget.moc"
-#include <qpainter.h>
-#include <qpixmap.h>
-#include <qimage.h>
-#include <math.h>
+
+#if defined(_MSC_VER)
+#define _USE_MATH_DEFINES
+#endif
+#include <cmath>
+#include <QPainter>
 
 #include "sccolor.h"
+#include "sccolorengine.h"
 
 
-ColorWheel::ColorWheel(QWidget * parent, const char * name) : QLabel(parent, name, WNoAutoErase)
+ColorWheel::ColorWheel(QWidget * parent, const char * name) : QLabel(parent)
 {
+	setObjectName(name);
+	pointList.clear();
+	currentDoc = NULL;
+	currentColorSpace = colorModelRGB;
 	baseAngle = 0;
 	angleShift = 270;
 	widthH = heightH = 150;
@@ -28,11 +34,14 @@ ColorWheel::ColorWheel(QWidget * parent, const char * name) : QLabel(parent, nam
 	{
 		QColor c;
 		c.setHsv(i, 255, 255);
-		colorMap[mapIndex] = c;
-		++mapIndex;
+		ScColor col;
+		col.fromQColor(c);
+		colorMap[mapIndex++] = col;
 		if (mapIndex > 359)
 			mapIndex = 0;
 	}
+	actualColor = colorMap[0];
+	trBaseColor = tr("Base Color");
 }
 
 void ColorWheel::mousePressEvent(QMouseEvent *e)
@@ -49,13 +58,26 @@ void ColorWheel::mouseReleaseEvent(QMouseEvent *e)
 {
 	baseAngle = valueFromPoint(e->pos());
 	actualColor = colorMap[baseAngle];
+	actualColor = ScColorEngine::convertToModel(actualColor, currentDoc, currentColorSpace);
 	emit clicked(e->button(), e->pos());
+	update();
 }
 
 void ColorWheel::paintEvent(QPaintEvent *)
 {
 	paintWheel();
 	paintCenterSample();
+	makeColors();
+	// clear marks
+	for (int i = 0; i < 360; ++i)
+		drawBorderPoint(i, false, true);
+	QList<PaintPoint>::const_iterator it;
+	for (it = pointList.constBegin(); it != pointList.constEnd(); ++it)
+		drawBorderPoint((*it).angle, (*it).base);
+}
+
+void ColorWheel::makeColors()
+{
 	if (currentType == Monochromatic)
 		makeMonochromatic();
 	if (currentType == Analogous)
@@ -75,7 +97,7 @@ void ColorWheel::paintCenterSample()
 	QPainter p;
 	p.begin(this);
 	p.setPen(QPen(Qt::black, 2));
-	p.setBrush(actualColor);
+	p.setBrush(ScColorEngine::getDisplayColor(actualColor, currentDoc ));
 	p.drawEllipse(widthH - 20, heightH - 20, 40, 40);
 	p.end();
 }
@@ -83,7 +105,8 @@ void ColorWheel::paintCenterSample()
 void ColorWheel::paintWheel()
 {
 	int h, s, v;
-	actualColor.hsv(&h, &s, &v);
+	QColor col(ScColorEngine::getDisplayColor(actualColor, currentDoc ));
+	col.getHsv(&h, &s, &v);
 	int width = this->width();
 	int height = this->height();
 	QPainter p;
@@ -97,7 +120,7 @@ void ColorWheel::paintWheel()
 	widthH = width / 2;
 	for (int i = 0; i < 360; ++i)
 	{
-		QWMatrix matrix;
+		QMatrix matrix;
 		matrix.translate(widthH, heightH);
 		matrix.rotate((float)i);
 		p.setWorldMatrix(matrix);
@@ -123,49 +146,67 @@ QString ColorWheel::getTypeDescription(MethodType aType)
 	return "n/a";
 }
 
+ScColor ColorWheel::colorByAngle(int angle)
+{
+	while (angle > 359)
+		angle -= 359;
+	while (angle < 0)
+		angle += 359;
+	return colorSpaceColor(colorMap[angle]);
+}
+
 ScColor ColorWheel::sampleByAngle(int angle)
 {
 	while (angle > 359)
 		angle -= 359;
 	while (angle < 0)
 		angle += 359;
-	drawBorderPoint(angle);
-	return cmykColor(colorMap[angle]);
+	//drawBorderPoint(angle);
+	PaintPoint p;
+	p.angle = angle;
+	p.base = false;
+	pointList.append(p);
+	return colorSpaceColor(colorMap[angle]);
 }
 
-ScColor ColorWheel::cmykColor(QColor col)
+ScColor ColorWheel::colorSpaceColor(ScColor col)
 {
-	ScColor c = ScColor();
-	/* Dirty Hack to avoid Color Managed RGB -> CMYK conversion */
-	c.setSpotColor(true);
-	int h, sm, vm, s, v;
-	actualColor.getHsv(&h, &s, &v);
-	col.getHsv(&h, &sm, &vm);
-	col.setHsv(h, s, v);
-	c.fromQColor(col);
-	c.setColorModel(colorModelCMYK);
-	c.setSpotColor(false);
-	return c;
+	QColor newcol;
+	ScColor ret;
+	int h, s, v;
+
+	ScColorEngine::getRGBColor(col, currentDoc).getHsv(&h, &s, &v);
+	newcol.setHsv(h, s, v);
+	ret.fromQColor(newcol);
+	ret = ScColorEngine::convertToModel(ret, currentDoc, currentColorSpace);
+	return ret;
 }
 
 void ColorWheel::baseColor()
 {
-	clearBorder();
-	drawBorderPoint(baseAngle, true);
-	paintCenterSample();
+	//clearBorder();
+	pointList.clear();
+	//drawBorderPoint(baseAngle, true);
+	PaintPoint p;
+	p.angle = baseAngle;
+	p.base = true;
+	pointList.append(p);
+	//paintCenterSample();
 	colorList.clear();
-	colorList[ tr("Base Color")] = cmykColor(actualColor);
+	colorList[trBaseColor] = colorSpaceColor(actualColor);
 }
 
 void ColorWheel::makeMonochromatic()
 {
 	baseColor();
-	ScColor l = ScColor();
-	l.fromQColor(actualColor.light());
-	ScColor d = ScColor();
-	d.fromQColor(actualColor.dark());
+	QColor col(ScColorEngine::getRGBColor(actualColor, currentDoc));
+	ScColor l;
+	l.fromQColor(col.light());
+	l = ScColorEngine::convertToModel(l, currentDoc, currentColorSpace);
 	colorList[tr("Monochromatic Light")] = l;
-	colorList[tr("Monochromatic Dark")] = d;
+	l.fromQColor(col.dark());
+	l = ScColorEngine::convertToModel(l, currentDoc, currentColorSpace);
+	colorList[tr("Monochromatic Dark")] = l;
 	currentType = Monochromatic;
 }
 
@@ -211,11 +252,11 @@ void ColorWheel::makeTetradic()
 	currentType = Tetradic;
 }
 
-void ColorWheel::clearBorder()
-{
-	for (int i = 0; i < 360; ++i)
-		drawBorderPoint(i, false, true);
-}
+// void ColorWheel::clearBorder()
+// {
+// 	for (int i = 0; i < 360; ++i)
+// 		drawBorderPoint(i, false, true);
+// }
 
 void ColorWheel::drawBorderPoint(int angle, bool base, bool clear)
 {
@@ -253,35 +294,37 @@ int ColorWheel::valueFromPoint(const QPoint & p) const
 	if ( a < M_PI/-2 )
 		a = a + M_PI * 2;
 
-	int dist = 0;
 	int minv = 0, maxv = 359;
 	int r = maxv - minv;
 	int val;
 
 	val = (int)(0.5 + minv + r * (M_PI * 3/2 -a) / (2 * M_PI));
 
-	if ( dist > 0 )
-		val -= dist;
-
 	return val;
 }
 
-bool ColorWheel::recomputeColor(QColor col)
+bool ColorWheel::recomputeColor(ScColor col)
 {
+	int angle;
 	int origh, origs, origv;
 	ColorMap::iterator it;
+	QColor c(ScColorEngine::getRGBColor(col, currentDoc));
+	QColor act(ScColorEngine::getRGBColor(actualColor, currentDoc));
 
-	col.hsv(&origh, &origs, &origv);
-	for (it = colorMap.begin(); it != colorMap.end(); ++it)
+	c.getHsv(&origh, &origs, &origv);
+	angle = origh + angleShift;
+	if (angle > 359)
+		angle -= 360;
+	if (colorMap.contains(angle))
 	{
 		int tmph, tmps, tmpv;
-		it.data().hsv(&tmph, &tmps, &tmpv);
-		if (origh == tmph)
-		{
-			actualColor.setHsv(tmph, origs, origv);
-			baseAngle = it.key();
-			return true;
-		}
+		QColor col(ScColorEngine::getRGBColor(colorMap[angle], currentDoc));
+		col.getHsv(&tmph, &tmps, &tmpv);
+		act.setHsv(tmph , origs, origv);
+		actualColor.fromQColor(act);
+		actualColor = ScColorEngine::convertToModel(actualColor, currentDoc, currentColorSpace);
+		baseAngle = angle;
+		return true;
 	}
 	return false;
 }
