@@ -11,12 +11,12 @@ for which a new license (GPL+exception) is in place.
 #include "csvdia.h"
 #include <gtwriter.h>
 #include <gtparagraphstyle.h>
-#include <qcstring.h>
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qtextcodec.h>
+#include <QByteArray>
+#include <QFile>
+#include <QFileInfo>
+#include <QString>
+#include <QStringList>
+#include <QTextCodec>
 
 extern "C" PLUGIN_API void GetText(QString filename, QString encoding, bool textOnly, gtWriter *writer);
 
@@ -50,7 +50,7 @@ private:
 	gtParagraphStyle *pstyleHeader;
 	void loadFile();
 	void parseLine(const QString& line, bool isHeader);
-	QString toUnicode(const QString& text);
+	QString toUnicode(const QByteArray& rawText);
 	void setupPStyles();
 	void setupTabulators();
 };
