@@ -24,11 +24,10 @@ for which a new license (GPL+exception) is in place.
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#include <qobject.h>
+#include <QObject>
+#include <QByteArray>
 #include "htmlreader.h"
 
-#ifdef HAVE_XML
-
 #include "scribusstructs.h"
 #include "gtmeasure.h"
 
@@ -48,6 +47,9 @@ HTMLReader::HTMLReader(gtParagraphStyle *ps, gtWriter *w, bool textOnly)
 	inH1 = false;
 	inH2 = false;
 	inH3 = false;
+	inH4 = false;
+	inH5 = false;
+	inH6 = false;
 	inA = false;
 	inCode = false;
 	inBody = false;
@@ -80,6 +82,18 @@ void HTMLReader::initPStyles()
 	pstyleli->setName("HTML_li_level-0");
 	listStyles.push_back(pstyleli);
 	nextItemNumbers.push_back(1);
+	pstyleh6 = new gtParagraphStyle(*pstyle);
+	pstyleh6->getFont()->setSize(pstyle->getFont()->getSize() + 2.5);
+	pstyleh6->getFont()->setWeight(BOLD);
+	pstyleh6->setSpaceAbove(2.5);
+	pstyleh6->setSpaceBelow(1.25);
+	pstyleh6->setName("HTML_h6");
+	pstyleh5 = new gtParagraphStyle(*pstyle);
+	pstyleh5->getFont()->setSize(pstyle->getFont()->getSize() + 5);
+	pstyleh5->getFont()->setWeight(BOLD);
+	pstyleh5->setSpaceAbove(5.0);
+	pstyleh5->setSpaceBelow(2.5);
+	pstyleh5->setName("HTML_h5");
 	pstyleh4 = new gtParagraphStyle(*pstyle);
 	pstyleh4->getFont()->setSize(pstyle->getFont()->getSize() + 10);
 	pstyleh4->getFont()->setWeight(BOLD);
@@ -118,7 +132,7 @@ void HTMLReader::startElement(void*, const xmlChar * fullname, const xmlChar **
 {
 	elemJustStarted = true;
 	elemJustFinished = false;
-	QString name(QString((const char*) fullname).lower());
+	QString name(QString((const char*) fullname).toLower());
 	QXmlAttributes attrs;
 	if (atts)
 	{
@@ -183,6 +197,10 @@ bool HTMLReader::startElement(const QString&, const QString&, const QString &nam
 		inH3 = true;
 	else if (name == "h4")
 		inH4 = true;
+	else if (name == "h5")
+		inH5 = true;
+	else if (name == "h6")
+		inH6 = true;
 	else if ((name == "b") || (name == "strong"))
 		setBoldFont();
 	else if ((name == "i") || (name == "em"))
@@ -251,13 +269,13 @@ bool HTMLReader::characters(const QString &ch)
 				tmp = tmp.right(tmp.length() - 2);
 		}
 		else
-			tmp = tmp.simplifyWhiteSpace();
+			tmp = tmp.simplified();
 
 		if (!lastCharWasSpace)
 			if (fcis)
 				tmp = " " + tmp;
 
-		if (lcis)
+		if (lcis && !(fcis && tmp.length() <= 1))
 			tmp = tmp + " ";
 		lastCharWasSpace = lcis;
 		if ((inLI) && (!addedLI))
@@ -273,7 +291,7 @@ bool HTMLReader::characters(const QString &ch)
 		}
 
 		if (noFormatting)
-			writer->append(tmp);
+			writer->appendUnstyled(tmp);
 		else if (inP)
 			writer->append(tmp, pstylep);
 		else if (inLI)
@@ -288,6 +306,10 @@ bool HTMLReader::characters(const QString &ch)
 			writer->append(tmp, pstyleh3);
 		else if (inH4)
 			writer->append(tmp, pstyleh4);
+		else if (inH5)
+			writer->append(tmp, pstyleh5);
+		else if (inH6)
+			writer->append(tmp, pstyleh6);
 		else if (inCenter)
 			writer->append(tmp, pstylec);
 		else if (inCode)
@@ -304,7 +326,7 @@ void HTMLReader::endElement(void*, const xmlChar * name)
 {
 	elemJustStarted = false;
 	elemJustFinished = true;
-	QString nname(QString((const char*) name).lower());
+	QString nname(QString((const char*) name).toLower());
 	hreader->endElement(NULL, NULL, nname);
 }
 
@@ -323,8 +345,8 @@ bool HTMLReader::endElement(const QString&, const QString&, const QString &name)
 	else if (name == "a")
 	{
 		toggleEffect(UNDERLINE);
-		if ((!href.isEmpty()) && ((href.find("//") != -1) ||
-		    (href.find("mailto:") != -1) || (href.find("www") != -1)))
+		if ((!href.isEmpty()) && ((href.indexOf("//") != -1) ||
+		    (href.indexOf("mailto:") != -1) || (href.indexOf("www") != -1)))
 		{
 			href = href.remove("mailto:");
 			writer->append(QString(" [%1]").arg(extIndex), pstyle);
@@ -407,22 +429,32 @@ bool HTMLReader::endElement(const QString&, const QString&, const QString &name)
 	else if (name == "h1")
 	{
 		inH1 = false;
-		writer->append("\n");
+		writer->append("\n", pstyleh1);
 	}
 	else if (name == "h2")
 	{
 		inH2 = false;
-		writer->append("\n");
+		writer->append("\n", pstyleh2);
 	}
 	else if (name == "h3")
 	{
 		inH3 = false;
-		writer->append("\n");
+		writer->append("\n", pstyleh3);
 	}
 	else if (name == "h4")
 	{
 		inH4 = false;
-		writer->append("\n");
+		writer->append("\n", pstyleh4);
+	}
+	else if (name == "h5")
+	{
+		inH5 = false;
+		writer->append("\n", pstyleh5);
+	}
+	else if (name == "h6")
+	{
+		inH6 = false;
+		writer->append("\n", pstyleh6);
 	}
 	else if ((name == "b") || (name == "strong"))
 		unSetBoldFont();
@@ -460,6 +492,8 @@ void HTMLReader::toggleEffect(FontEffect e)
 	pstyleh2->getFont()->toggleEffect(e);
 	pstyleh3->getFont()->toggleEffect(e);
 	pstyleh4->getFont()->toggleEffect(e);
+	pstyleh5->getFont()->toggleEffect(e);
+	pstyleh6->getFont()->toggleEffect(e);
 	pstylecode->getFont()->toggleEffect(e);
 	pstylep->getFont()->toggleEffect(e);
 	pstylepre->getFont()->toggleEffect(e);
@@ -475,6 +509,8 @@ void HTMLReader::setItalicFont()
 	pstyleh2->getFont()->setSlant(ITALIC);
 	pstyleh3->getFont()->setSlant(ITALIC);
 	pstyleh4->getFont()->setSlant(ITALIC);
+	pstyleh5->getFont()->setSlant(ITALIC);
+	pstyleh6->getFont()->setSlant(ITALIC);
 	pstylecode->getFont()->setSlant(ITALIC);
 	pstylep->getFont()->setSlant(ITALIC);
 	pstylepre->getFont()->setSlant(ITALIC);
@@ -490,6 +526,8 @@ void HTMLReader::unsetItalicFont()
 	pstyleh2->getFont()->setSlant(defaultSlant);
 	pstyleh3->getFont()->setSlant(defaultSlant);
 	pstyleh4->getFont()->setSlant(defaultSlant);
+	pstyleh5->getFont()->setSlant(defaultSlant);
+	pstyleh6->getFont()->setSlant(defaultSlant);
 	pstylecode->getFont()->setSlant(defaultSlant);
 	pstylep->getFont()->setSlant(defaultSlant);
 	pstylepre->getFont()->setSlant(defaultSlant);
@@ -505,6 +543,8 @@ void HTMLReader::setBlueFont()
 	pstyleh2->getFont()->setColor("Blue");
 	pstyleh3->getFont()->setColor("Blue");
 	pstyleh4->getFont()->setColor("Blue");
+	pstyleh5->getFont()->setColor("Blue");
+	pstyleh6->getFont()->setColor("Blue");
 	pstylecode->getFont()->setColor("Blue");
 	pstylep->getFont()->setColor("Blue");
 	pstylepre->getFont()->setColor("Blue");
@@ -520,6 +560,8 @@ void HTMLReader::setDefaultColor()
 	pstyleh2->getFont()->setColor(defaultColor);
 	pstyleh3->getFont()->setColor(defaultColor);
 	pstyleh4->getFont()->setColor(defaultColor);
+	pstyleh5->getFont()->setColor(defaultColor);
+	pstyleh6->getFont()->setColor(defaultColor);
 	pstylecode->getFont()->setColor(defaultColor);
 	pstylep->getFont()->setColor(defaultColor);
 	pstylepre->getFont()->setColor(defaultColor);
@@ -550,10 +592,10 @@ void HTMLReader::unSetBoldFont()
 void HTMLReader::parse(QString filename)
 {
 #if defined(_WIN32)
-	QString fname = QDir::convertSeparators(filename);
-	QCString fn = (qWinVersion() & Qt::WV_NT_based) ? fname.utf8() : fname.local8Bit();
+	QString fname = QDir::toNativeSeparators(filename);
+	QByteArray fn = (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based) ? fname.toUtf8() : fname.toLocal8Bit();
 #else
-	QCString fn(filename.local8Bit());
+	QByteArray fn(filename.toLocal8Bit());
 #endif
 	elemJustStarted = elemJustFinished = false;
 	htmlSAXParseFile(fn.data(), NULL, mySAXHandler, NULL);
@@ -624,10 +666,11 @@ HTMLReader::~HTMLReader()
 	delete pstyleh2;
 	delete pstyleh3;
 	delete pstyleh4;
+	delete pstyleh5;
+	delete pstyleh6;
 	delete pstylecode;
 	delete pstylep;
 	delete pstylepre;
 	hreader = NULL;
 }
 
-#endif // HAVE_XML
