@@ -1,6 +1,12 @@
+/*
+For general Scribus (>=1.3.2) copyright and licensing information please refer
+to the COPYING file provided with the program. Following this notice may exist
+a copyright and/or license notice that predates the release of Scribus 1.3.2
+for which a new license (GPL+exception) is in place.
+*/
 /***************************************************************************
  *   Copyright (C) 2004 by Riku Leino                                      *
- *   riku.leino@gmail.com                                                      *
+ *   tsoots@gmail.com                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -20,19 +26,17 @@
 #ifndef HTMLREADER_H
 #define HTMLREADER_H
 
-#include "config.h"
-
-#ifdef HAVE_XML
+#include "scconfig.h"
 
 #include <vector>
-
 #include <libxml/HTMLparser.h>
 
-#include <qstring.h>
-#include <qxml.h>
+#include <QString>
+#include <QXmlAttributes>
 
 #include <gtparagraphstyle.h>
 #include <gtwriter.h>
+
 /*! \brief Parse and import a HTML file.
 Supported tags: P, CENTER, BR, A, UL, OL, LI, H1, H2, H3, H4,
 B, STRONG, I, EM, CODE, BODY, PRE, IMG, SUB, SUP, DEL, INS, U,
@@ -60,6 +64,8 @@ private:
 	gtParagraphStyle *pstyleh2;
 	gtParagraphStyle *pstyleh3;
 	gtParagraphStyle *pstyleh4;
+	gtParagraphStyle *pstyleh5;
+	gtParagraphStyle *pstyleh6;
 	gtParagraphStyle *pstylecode;
 	gtParagraphStyle *pstylep;
 	gtParagraphStyle *pstylepre;
@@ -73,12 +79,16 @@ private:
 	bool inH2;
 	bool inH3;
 	bool inH4;
+	bool inH5;
+	bool inH6;
 	bool inA;
 	bool inCenter;
 	bool inCode;
 	bool inBody;
 	bool inPre;
 	bool inP;
+	static bool elemJustStarted;
+	static bool elemJustFinished;
 	bool lastCharWasSpace;
 	bool noFormatting;
 	void initPStyles();
@@ -103,6 +113,4 @@ public:
 	bool characters(const QString &ch);
 };
 
-#endif // HAVE_XML
-
 #endif
