@@ -1,3 +1,9 @@
+/*
+For general Scribus (>=1.3.2) copyright and licensing information please refer
+to the COPYING file provided with the program. Following this notice may exist
+a copyright and/or license notice that predates the release of Scribus 1.3.2
+for which a new license (GPL+exception) is in place.
+*/
 /***************************************************************************
  *   Copyright (C) 2004 by Riku Leino                                      *
  *   tsoots@gmail.com                                                      *
@@ -18,9 +24,10 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <QGlobalStatic>
+#include <QByteArray>
 #include "contentreader.h"
-
-#ifdef HAVE_XML
+#include "scribusstructs.h"
 
 ContentReader* ContentReader::creader = NULL;
 
@@ -35,9 +42,12 @@ ContentReader::ContentReader(QString documentName, StyleReader *s, gtWriter *w,
 	importTextOnly = textOnly;
 	defaultStyle = NULL;
 	currentStyle = NULL;
-	append = false;
 	inList = false;
+	inNote = false;
+	inAnnotation = false;
+	inNoteBody = false;
 	inSpan = false;
+	append = 0;
 	listIndex = 0;
 	listLevel = 0;
 	currentList = "";
@@ -50,7 +60,7 @@ bool ContentReader::startElement(const QString&, const QString&, const QString &
 {
 	if ((name == "text:p") || (name == "text:h"))
 	{
-		append = true;
+		++append;
 		QString name = "";
 		for (int i = 0; i < attrs.count(); ++i)
 		{
@@ -68,7 +78,7 @@ bool ContentReader::startElement(const QString&, const QString&, const QString &
 		else
 		{
 			gtStyle *tmp = sreader->getStyle(getName());
-			if ((tmp->getName()).find("default-style") != -1)
+			if ((tmp->getName()).indexOf("default-style") != -1)
 				getStyle();
 			else
 				currentStyle = tmp;
@@ -88,7 +98,7 @@ bool ContentReader::startElement(const QString&, const QString&, const QString &
 			}
 		}
 		gtStyle *tmp = sreader->getStyle(getName());
-		if ((tmp->getName()).find("default-style") != -1)
+		if ((tmp->getName()).indexOf("default-style") != -1)
 			getStyle();
 		else
 			currentStyle = tmp;
@@ -106,15 +116,25 @@ bool ContentReader::startElement(const QString&, const QString&, const QString &
 		}
 		currentStyle = sreader->getStyle(QString(currentList + "_%1").arg(listLevel));
 		currentListStyle = sreader->getList(currentList);
-		currentListStyle->setLevel(listLevel);
+		if (currentListStyle)
+			currentListStyle->setLevel(listLevel);
 		styleNames.clear();
 		styleNames.push_back(QString(currentList + "_%1").arg(listLevel));
 	}
 	else if (name == "text:list-item")
 	{
-		currentListStyle->advance();
-		write(currentListStyle->bullet());
+		if (currentListStyle)
+		{
+			currentListStyle->advance();
+			write(currentListStyle->bullet());
+		}
 	}
+	else if (name == "office:annotation")
+		inAnnotation = true;
+	else if (name == "text:note")
+		inNote = true;
+	else if (name == "text:note-body")
+		inNoteBody = true;
 	else if (name == "style:style")
 	{
 		QString sname = "";
@@ -168,8 +188,9 @@ bool ContentReader::characters(const QString &ch)
 	QString tmp = ch;
 	tmp = tmp.remove("\n");
 	tmp = tmp.remove(""); // Remove all OO.o hyphenation chars
-	tmp = tmp.replace(QChar(160), QChar(29)); // replace OO.o nbsp with Scribus nbsp
-	if (append)
+	// Unneeded as scribus now also use standard unicode non-breakable space
+	// tmp = tmp.replace(QChar(160), SpecialChars::NBSPACE); // replace OO.o nbsp with Scribus nbsp
+	if (append > 0)
 		write(tmp);
 	return true;
 }
@@ -178,27 +199,53 @@ bool ContentReader::endElement(const QString&, const QString&, const QString &na
 {
 	if ((name == "text:p") || (name == "text:h"))
 	{
+// 		qDebug("TPTH");
 		write("\n");
-		append = false;
-		if (inList)
-			styleNames.pop_back();
+		--append;
+		if (inList || inAnnotation || inNote || inNoteBody)
+		{
+			if(static_cast<int>(styleNames.size()) > 0)
+				styleNames.pop_back();
+		}
 		else
 			styleNames.clear();
 	}
 	else if (name == "text:span")
 	{
+// 		qDebug("TS");
 		inSpan = false;
 		currentStyle = pstyle;
 		if (styleNames.size() != 0)
 			styleNames.pop_back();	
 		currentStyle = sreader->getStyle(getName());
 	}
+	else if (name == "office:annotation")
+	{
+		inAnnotation = false;
+	}
+	else if (name == "text:note")
+	{
+// 		qDebug("TN");
+		inNote = false;
+	}
+	else if (name == "text:note-body")
+	{
+// 		qDebug("TNB");
+		inNoteBody = false;
+	}
 	else if (name == "text:line-break")
-		write(QChar(28));
+	{
+// 		qDebug("TLB");
+		write(SpecialChars::LINEBREAK);
+	}
 	else if (name == "text:tab")
+	{
+// 		qDebug("TT");
 		write("\t");
+	}
 	else if (name == "text:list")
 	{
+// 		qDebug("TL");
 		--listLevel;
 		styleNames.clear();
 		if (listLevel == 0)
@@ -211,13 +258,16 @@ bool ContentReader::endElement(const QString&, const QString&, const QString &na
 		{
 			currentStyle = sreader->getStyle(QString(currentList + "_%1").arg(listLevel));
 			styleNames.push_back(QString(currentList + "_%1").arg(listLevel));
-			currentListStyle->resetLevel();
+			if (currentListStyle)
+				currentListStyle->resetLevel();
 			currentListStyle = sreader->getList(currentList);
-			currentListStyle->setLevel(listLevel);
+			if (currentListStyle)
+				currentListStyle->setLevel(listLevel);
 		}
 	}
 	else if ((name == "style:style") && (inT))
 	{
+// 		qDebug("SS");
 		inT = false;
 		tName = "";
 	}
@@ -226,20 +276,28 @@ bool ContentReader::endElement(const QString&, const QString&, const QString &na
 
 void ContentReader::write(const QString& text)
 {
-	/*qDebug(text); */
-	if (importTextOnly)
-		writer->append(text);
-	else if (inSpan)
-		writer->append(text, currentStyle, false);
-	else
-		writer->append(text, currentStyle);
+	if (!inNote && !inNoteBody && !inAnnotation) // Disable notes import for now
+	{
+		if (importTextOnly)
+			writer->appendUnstyled(text);
+		else if (inSpan)
+			writer->append(text, currentStyle, false);
+		else
+			writer->append(text, currentStyle);
+	}
 	lastStyle = currentStyle;
 }
 
 void ContentReader::parse(QString fileName)
 {
 	sreader->parse(fileName);
-	xmlSAXParseFile(cSAXHandler, fileName.ascii(), 1);
+#if defined(_WIN32)
+	QString fname = QDir::toNativeSeparators(fileName);
+	QByteArray fn = (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based) ? fname.toUtf8() : fname.toLocal8Bit();
+#else
+	QByteArray fn(fileName.toLocal8Bit());
+#endif
+	xmlSAXParseFile(cSAXHandler, fn.data(), 1);
 }
 
 xmlSAXHandler cSAXHandlerStruct = {
@@ -284,15 +342,14 @@ xmlSAXHandlerPtr cSAXHandler = &cSAXHandlerStruct;
 
 void ContentReader::startElement(void*, const xmlChar *fullname, const xmlChar ** atts)
 {
-	QString* name = new QString((const char*) fullname);
-	name = new QString(name->lower());
-	QXmlAttributes* attrs = new QXmlAttributes();
+	QString name(QString((const char*) fullname).toLower());
+	QXmlAttributes attrs;
 	if (atts)
 	{
 		for(const xmlChar** cur = atts; cur && *cur; cur += 2)
-			attrs->append(QString((char*)*cur), NULL, QString((char*)*cur), QString((char*)*(cur + 1)));
+			attrs.append(QString((char*)*cur), NULL, QString((char*)*cur), QString((char*)*(cur + 1)));
 	}
-	creader->startElement(NULL, NULL, *name, *attrs);
+	creader->startElement(NULL, NULL, name, attrs);
 }
 
 void ContentReader::characters(void*, const xmlChar *ch, int len)
@@ -303,9 +360,8 @@ void ContentReader::characters(void*, const xmlChar *ch, int len)
 
 void ContentReader::endElement(void*, const xmlChar *name)
 {
-	QString *nname = new QString((const char*) name);
-	nname = new QString(nname->lower());
-	creader->endElement(NULL, NULL, *nname);
+	QString nname(QString((const char*) name).toLower());
+	creader->endElement(NULL, NULL, nname);
 }
 
 QString ContentReader::getName()
@@ -318,19 +374,23 @@ QString ContentReader::getName()
 
 void ContentReader::getStyle()
 {
-	gtParagraphStyle* par = NULL;
+	gtStyle *style = NULL, *tmp = NULL;
 	if (styleNames.size() == 0)
-		par = dynamic_cast<gtParagraphStyle*>(sreader->getStyle("default-style"));
+		style = sreader->getStyle("default-style");
 	else
-		par = dynamic_cast<gtParagraphStyle*>(sreader->getStyle(styleNames[0]));
-	gtParagraphStyle* tmp = new gtParagraphStyle(*par);
+		style = sreader->getStyle(styleNames[0]);
+	assert (style != NULL);
+	gtParagraphStyle* par = dynamic_cast<gtParagraphStyle*>(style);
+	if (par)
+		tmp = new gtParagraphStyle(*par);
+	else
+		tmp = new gtStyle(*style);
 	for (uint i = 1; i < styleNames.size(); ++i)
 	{
 		Properties& p = tmap[styleNames[i]];
 		for (uint j = 0; j < p.size(); ++j)
 			sreader->updateStyle(tmp, sreader->getStyle(styleNames[i - 1]), p[j].first, p[j].second);
 	}
-
 	currentStyle = tmp;
 	sreader->setStyle(getName(), tmp);
 }
@@ -340,5 +400,3 @@ ContentReader::~ContentReader()
 	creader = NULL;
 	delete defaultStyle;
 }
-
-#endif // HAVE_XML
