@@ -1,6 +1,12 @@
+/*
+For general Scribus (>=1.3.2) copyright and licensing information please refer
+to the COPYING file provided with the program. Following this notice may exist
+a copyright and/or license notice that predates the release of Scribus 1.3.2
+for which a new license (GPL+exception) is in place.
+*/
  /***************************************************************************
   *   Copyright (C) 2004 by Riku Leino                                      *
-  *   riku.leino@gmail.com                                                      *
+  *   tsoots@gmail.com                                                      *
   *                                                                         *
   *   This program is free software; you can redistribute it and/or modify  *
   *   it under the terms of the GNU General Public License as published by  *
@@ -20,12 +26,12 @@
  
  #include "stylereader.h"
  
- #ifdef HAVE_XML
- 
+ #include <scribusstructs.h>
  #include <gtmeasure.h>
  #include <gtparagraphstyle.h>
  #include <gtframestyle.h>
  #include <gtfont.h>
+#include <QByteArray>
  
  StyleReader* StyleReader::sreader = NULL;
  
@@ -40,7 +46,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	writer       = w;
  	importTextOnly = textOnly;
  	usePrefix    = prefix;
- 	packStyles   = combineStyles;
+	packStyles   = combineStyles;
  	currentStyle = NULL;
  	parentStyle  = NULL;
  	inList       = false;
@@ -56,12 +62,14 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  		styleProperties(attrs);
  	else if (name == "style:style")
  	{
-		if (!defaultStyleCreated)
+		/*if (!defaultStyleCreated)
 		{
-			currentStyle = new gtParagraphStyle(*(writer->getDefaultStyle()));
+			gtParagraphStyle* pstyle = new gtParagraphStyle(*(writer->getDefaultStyle()));
+			pstyle->setDefaultStyle(true);
+			currentStyle = dynamic_cast<gtStyle*>(pstyle);
 			currentStyle->setName("default-style");
 			defaultStyleCreated = true;
-		}
+		}*/
  		styleStyle(attrs);
  	}
  	else if (name == "style:tab-stop")
@@ -100,6 +108,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  				if (plist->target() == "paragraph")
  				{
  					pstyle = dynamic_cast<gtParagraphStyle*>(plist);
+					assert(pstyle != NULL);
  					gtParagraphStyle* tmp = new gtParagraphStyle(*pstyle);
  					currentStyle = tmp;
  				}
@@ -127,6 +136,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  					if (ok)
  					{
  						gtParagraphStyle* s = dynamic_cast<gtParagraphStyle*>(currentStyle);
+						assert(s != NULL);
  						s->setDropCapHeight(dh);
  						s->setDropCap(true);
  					}
@@ -152,7 +162,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  				style += attrs.value(i) + " ";
  		}
  		QString name = family + " " + style;
- 		name = name.simplifyWhiteSpace();
+		name = name.simplified();
  		fonts[key] = name;
  	}
  	return true;
@@ -165,7 +175,9 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  		if (attrs.localName(i) == "style:family")
  			if (attrs.value(i) == "paragraph")
  			{
- 				currentStyle = new gtParagraphStyle(*(writer->getDefaultStyle()));
+ 				gtParagraphStyle* pstyle = new gtParagraphStyle(*(writer->getDefaultStyle()));
+				pstyle->setDefaultStyle(true);
+				currentStyle = dynamic_cast<gtStyle*>(pstyle);
 				currentStyle->setName("default-style");
  				readProperties = true;
 				defaultStyleCreated = true;
@@ -210,7 +222,8 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  			if (parentStyle->target() == "paragraph")
  			{
  				ppstyle = dynamic_cast<gtParagraphStyle*>(parentStyle);
- 				pstyle->setLineSpacing(getSize(attrs.value(i), writer->getPreferredLineSpacing(currentStyle->getFont()->getSize())));
+				assert(ppstyle != NULL);
+ 				ppstyle->setLineSpacing(getSize(attrs.value(i), writer->getPreferredLineSpacing(currentStyle->getFont()->getSize())));
  			}
  		}
  		else if (attrs.localName(i) == "fo:color")
@@ -240,15 +253,15 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  			currentStyle->getFont()->setColor("White");
  		}
  		else if (attrs.localName(i) == "fo:letter-spacing")
- 			currentStyle->getFont()->setKerning(getSize(attrs.value(i)));
+ 			currentStyle->getFont()->setKerning(static_cast<int>(getSize(attrs.value(i), -1.0)));
  		else if (attrs.localName(i) == "style:text-scale")
  			currentStyle->getFont()->setHscale(static_cast<int>(getSize(attrs.value(i), -1.0)));
  		else if ((attrs.localName(i) == "style:text-position") && 
- 		        (((attrs.value(i)).find("sub") != -1) || 
+ 		        (((attrs.value(i)).indexOf("sub") != -1) || 
  				(((attrs.value(i)).left(1) == "-") && ((attrs.value(i)).left(1) != "0"))))
  			currentStyle->getFont()->toggleEffect(SUBSCRIPT);
  		else if ((attrs.localName(i) == "style:text-position") && 
- 		        (((attrs.value(i)).find("super") != -1) || 
+ 		        (((attrs.value(i)).indexOf("super") != -1) || 
  				(((attrs.value(i)).left(1) != "-") && ((attrs.value(i)).left(1) != "0"))))
  			currentStyle->getFont()->toggleEffect(SUPERSCRIPT);
  		else if ((attrs.localName(i) == "fo:margin-top") && (pstyle != NULL))
@@ -276,7 +289,8 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  		else if ((attrs.localName(i) == "style:justify-single-word") && (pstyle != NULL))
  			force = attrs.value(i);
  	}
- 	if (align != NULL)
+	// Qt4 NULL->isNull()
+ 	if (!align.isNull())
  	{
  		if (align == "end")
  			pstyle->setAlignment(RIGHT);
@@ -298,8 +312,21 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  {
  	QString name = "";
  	QString listName = NULL;
+	bool setDefaultStyle = false;
  	bool isParaStyle = false;
  	bool create = true;
+
+	if (!defaultStyleCreated)
+	{
+		gtParagraphStyle* pstyle = new gtParagraphStyle(*(writer->getDefaultStyle()));
+		pstyle->setDefaultStyle(true);
+		currentStyle = dynamic_cast<gtStyle*>(pstyle);
+		currentStyle->setName("default-style");
+		setDefaultStyle     = true;
+		defaultStyleCreated = true;
+		parentStyle  = currentStyle;
+	}
+
  	for (int i = 0; i < attrs.count(); ++i)
  	{
  		if (attrs.localName(i) == "style:family")
@@ -347,6 +374,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  			if (parentStyle->target() == "paragraph")
  			{
  				tmpP = dynamic_cast<gtParagraphStyle*>(parentStyle);
+				assert(tmpP != NULL);
  				gtParagraphStyle* tmp = new gtParagraphStyle(*tmpP);
  // 				tmp->setAutoLineSpacing(true);
  				currentStyle = tmp;
@@ -357,7 +385,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  // 				tmp->setAutoLineSpacing(true);
  				currentStyle = tmp;
  			}
- 			if (listName != NULL)
+ 			if (!listName.isNull())
  			{
  				listParents[listName] = currentStyle;
  			}
@@ -366,6 +394,12 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  			currentStyle = new gtStyle(*parentStyle);
  
  		currentStyle->setName(name);
+		if (setDefaultStyle)
+		{
+			gtParagraphStyle* tmp = dynamic_cast<gtParagraphStyle*>(currentStyle);
+			if (tmp)
+				tmp->setDefaultStyle(true);
+		}
  	}
  	else
  		currentStyle = NULL;
@@ -376,6 +410,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	if (currentStyle->target() == "paragraph")
  	{
  		gtParagraphStyle* pstyle = dynamic_cast<gtParagraphStyle*>(currentStyle);
+		assert(pstyle != NULL);
  		QString pos = NULL;
  		QString type = NULL;
  		for (int i = 0; i < attrs.count(); ++i)
@@ -386,9 +421,9 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  				type = attrs.value(i);
  				
  		}
- 		if (pos != NULL)
+ 		if (!pos.isNull())
  		{
- 			if (type == NULL)
+ 			if (!type.isNull())
  				type = "left";
  			double posd = getSize(pos);
  			if (type == "left")
@@ -432,7 +467,29 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  
  void StyleReader::parse(QString fileName)
  {
- 	xmlSAXParseFile(sSAXHandler, fileName.ascii(), 1);
+#if defined(_WIN32)
+	QString fname = QDir::toNativeSeparators(fileName);
+	QByteArray fn = (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based) ? fname.toUtf8() : fname.toLocal8Bit();
+#else
+	QByteArray fn(fileName.toLocal8Bit());
+#endif
+ 	xmlSAXParseFile(sSAXHandler, fn.data(), 1);
+ }
+
+ gtStyle* StyleReader::getDefaultStyle(void)
+ {
+	 gtStyle* defStyle = writer->getDefaultStyle();
+	 StyleMap::Iterator it, itEnd = styles.end();
+	 for (it = styles.begin(); it != itEnd; ++it)
+	 {
+		 gtParagraphStyle *pStyle = dynamic_cast<gtParagraphStyle*> (it.value());
+		 if (pStyle && pStyle->isDefaultStyle())
+		 {
+			 defStyle = pStyle;
+			 break;
+		 }
+	 }
+	 return defStyle;
  }
  
  gtStyle* StyleReader::getStyle(const QString& name)
@@ -441,14 +498,13 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	{
  		gtStyle* tmp = styles[name];
  		QString tname = tmp->getName();
- 		if ((tname.find(docname) == -1) && (usePrefix))
+ 		if ((tname.indexOf(docname) == -1) && (usePrefix))
  			tmp->setName(docname + "_" + tname);
  
  		return tmp;
  	}
  	else
-		return styles["default-style"];
-
+		return getDefaultStyle();
  }
  
  void StyleReader::setStyle(const QString& name, gtStyle* style)
@@ -458,6 +514,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	if ((style->target() == "paragraph") && (packStyles))
  	{
  		s = dynamic_cast<gtParagraphStyle*>(style);
+		assert(s != NULL);
  		QString nameByAttrs = QString("%1-").arg(s->getSpaceAbove());
  		nameByAttrs += QString("%1-").arg(s->getSpaceBelow());
  		nameByAttrs += QString("%1-").arg(s->getLineSpacing());
@@ -467,12 +524,13 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  		nameByAttrs += QString("%1-").arg(s->hasDropCap());
  		nameByAttrs += QString("%1-").arg(s->getFont()->getColor());
  		nameByAttrs += QString("%1-").arg(s->getFont()->getStrokeColor());
- 		QValueList<double>* tmp = s->getTabValues();
+// TODO is this important ??
+/* 		QValueList<double>* tmp = s->getTabValues();
  		for (uint i = 0; i < tmp->count(); ++i)
  		{
  			double td = (*tmp)[i];
  			nameByAttrs += QString("%1-").arg(td);
- 		}
+ 		} */
  		if (attrsStyles.contains(nameByAttrs))
  		{
  			tname = attrsStyles[nameByAttrs]->getName();
@@ -494,7 +552,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	}
  	if (!styles.contains(name))
  	{
- 		if ((tname.find(docname) == -1) && (usePrefix))
+ 		if ((tname.indexOf(docname) == -1) && (usePrefix))
  			style->setName(docname + "_" + tname);
  		styles[name] = style;
  	}
@@ -515,9 +573,9 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	CounterMap::Iterator it;
  	for (it = pstyleCounts.begin(); it != pstyleCounts.end(); ++it)
  	{
- 		if (it.data() > count)
+ 		if (it.value() > count)
  		{
- 			count = it.data();
+ 			count = it.value();
  			fstyleName = it.key();
  		}
  	}
@@ -563,7 +621,8 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  		if (parent2Style->target() == "paragraph")
  		{
  			ppstyle = dynamic_cast<gtParagraphStyle*>(parent2Style);
- 			pstyle->setLineSpacing(getSize(value, writer->getPreferredLineSpacing(style->getFont()->getSize())));
+			assert(ppstyle != NULL);
+ 			ppstyle->setLineSpacing(getSize(value, writer->getPreferredLineSpacing(style->getFont()->getSize())));
  		}
  	}
  	else if (key == "fo:color")
@@ -587,15 +646,15 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  		style->getFont()->setColor("White");
  	}
  	else if (key == "fo:letter-spacing")
- 		style->getFont()->setKerning(getSize(value));
+ 		style->getFont()->setKerning(static_cast<int>(getSize(value, -1.0)));
  	else if (key == "style:text-scale")
  		style->getFont()->setHscale(static_cast<int>(getSize(value, -1.0)));
  	else if ((key == "style:text-position") && 
- 	        (((value).find("sub") != -1) || 
+ 	        (((value).indexOf("sub") != -1) || 
  			(((value).left(1) == "-") && ((value).left(1) != "0"))))
  		style->getFont()->toggleEffect(SUBSCRIPT);
  	else if ((key == "style:text-position") && 
- 	        (((value).find("super") != -1) || 
+ 	        (((value).indexOf("super") != -1) || 
  			(((value).left(1) != "-") && ((value).left(1) != "0"))))
  		style->getFont()->toggleEffect(SUPERSCRIPT);
  	else if ((key == "fo:margin-top") && (pstyle != NULL))
@@ -623,8 +682,9 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	else if ((key == "style:justify-single-word") && (pstyle != NULL))
  		force = value;
  
- 	if (align != NULL)
+ 	if (!align.isNull())
  	{
+		assert(pstyle);
  		if (align == "end")
  			pstyle->setAlignment(RIGHT);
  		else if (align == "center")
@@ -644,36 +704,42 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  double StyleReader::getSize(QString s, double parentSize)
  {
  	QString dbl = "0.0";
- 	QString lowerValue = s.lower();
+ 	QString lowerValue = s.toLower();
  	double ret = 0.0;
- 	if (lowerValue.find("pt") != -1)
+ 	if (lowerValue.indexOf("pt") != -1)
  	{
  		dbl = lowerValue.remove("pt");
  		ret = gtMeasure::d2d(dbl.toDouble(), SC_PT);
  	}
- 	else if (lowerValue.find("mm") != -1)
+ 	else if (lowerValue.indexOf("mm") != -1)
  	{
  		dbl = lowerValue.remove("mm");
  		ret = gtMeasure::d2d(dbl.toDouble(), SC_MM);
  	}
- 	else if (lowerValue.find("cm") != -1)
+ 	else if (lowerValue.indexOf("cm") != -1)
  	{
  		dbl = lowerValue.remove("cm");
  		ret = gtMeasure::d2d(dbl.toDouble() * 10, SC_MM);
  	}
- 	else if (lowerValue.find("in") != -1)
+ 	else if (lowerValue.indexOf("in") != -1)
  	{
  		dbl = lowerValue.remove("inch");
  		dbl = lowerValue.remove("in");
  		ret = gtMeasure::d2d(dbl.toDouble(), SC_IN);
  	}
- 	else if (lowerValue.find("pi") != -1)
+ 	else if (lowerValue.indexOf("pi") != -1)
  	{
  		dbl = lowerValue.remove("pica");
  		dbl = lowerValue.remove("pi");
  		ret = gtMeasure::d2d(dbl.toDouble(), SC_P);
  	}
- 	else if (lowerValue.find("%") != -1)
+ 	else if (lowerValue.indexOf("c") != -1)
+ 	{
+ 		dbl = lowerValue.remove("cicero");
+ 		dbl = lowerValue.remove("c");
+ 		ret = gtMeasure::d2d(dbl.toDouble(), SC_C);
+ 	}
+ 	else if (lowerValue.indexOf("%") != -1)
  	{
  		dbl = lowerValue.remove("%");
  		double factor = dbl.toDouble();
@@ -694,10 +760,10 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  	StyleMap::Iterator it;
  	for (it = styles.begin(); it != styles.end(); ++it)
  	{
- 		if (it.data())
+ 		if (it.value())
  		{
- 			delete it.data();
- 			it.data() = NULL;
+ 			delete it.value();
+ 			it.value() = NULL;
  		}
  	}
  }
@@ -745,7 +811,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  void StyleReader::startElement(void*, const xmlChar * fullname, const xmlChar ** atts)
  {
  	QString* name = new QString((const char*) fullname);
- 	name = new QString(name->lower());
+ 	name = new QString(name->toLower());
  	QXmlAttributes* attrs = new QXmlAttributes();
  	if (atts)
  	{
@@ -758,8 +824,7 @@ StyleReader::StyleReader(QString documentName, gtWriter *w,
  void StyleReader::endElement(void*, const xmlChar * name)
  {
  	QString *nname = new QString((const char*) name);
- 	nname = new QString(nname->lower());
+ 	nname = new QString(nname->toLower());
  	sreader->endElement(NULL, NULL, *nname);
  }
  
- #endif
