@@ -9,10 +9,14 @@ for which a new license (GPL+exception) is in place.
 
 #include "scribusapi.h"
 #include "scribusstructs.h"
+//Added by qt3to4:
+#include <QPixmap>
+#include <QList>
 
 class QString;
 class ScColor;
 class PrefsManager;
+class QDomDocument;
 
 class SCRIBUS_API ScPreview
 {
@@ -20,12 +24,11 @@ public:
 	ScPreview();
 	~ScPreview() {};
 	QPixmap createPreview(QString data);
-	void SetFarbe(QColor *tmp, QString farbe, int shad);
-	void DrawZeichenS(ScPainter *p, double xco, double yco, QString ch, QString ZFo, bool Reverse, int Style, int mode, int Siz);
+	void SetQColor(QColor *tmp, const QString& farbe, double shad);
+	void DrawZeichenS(ScPainter *p, double xco, double yco, QString ch, QString ZFo, bool Reverse, int Style, int mode, double Siz);
 	QMap<QString,ScColor> Farben;
 	QMap<QString,multiLine> MLineStyles;
-	QValueList<uint> Segments;
-	FT_Library   library;
+	QList<uint> Segments;
 protected:
 	PrefsManager* prefsManager;
 };
