@@ -21,22 +21,30 @@ for which a new license (GPL+exception) is in place.
  *                                                                         *
  ***************************************************************************/
 
-#include "vruler.h"
-#include "vruler.moc"
+#include <QCursor>
+//#include <QDebug>
+#include <QMouseEvent>
+#include <QPaintEvent>
+#include <QPainter>
+#include <QPixmap>
+#include <QPolygon>
+#include <QRect>
+#include <QRubberBand>
+
+#include <cmath>
+
+#include "canvasgesture_rulermove.h"
 #include "page.h"
-#include <qcursor.h>
-#include <qcolor.h>
-#include <qrect.h>
-#include <qpointarray.h>
+#include "prefsmanager.h"
 #include "scribus.h"
 #include "scribusdoc.h"
 #include "units.h"
-#include "prefsmanager.h"
+#include "vruler.h"
 
 #if QT_VERSION  > 0x030102
-	#define SPLITVC SplitHCursor
+	#define SPLITVC Qt::SplitHCursor
 #else
-	#define SPLITVC SplitVCursor
+	#define SPLITVC Qt::SplitVCursor
 #endif
 
 
@@ -44,42 +52,56 @@ for which a new license (GPL+exception) is in place.
 Vruler::Vruler(ScribusView *pa, ScribusDoc *doc) : QWidget(pa)
 {
 	prefsManager=PrefsManager::instance();
-	setEraseColor(QColor(255,255,255));
+	setBackgroundRole(QPalette::Window);
+	setAutoFillBackground(true);
+	QPalette palette;
+	palette.setBrush(QPalette::Window, QColor(255, 255, 255));
+	setPalette(palette);
 	currDoc = doc;
 	currView = pa;
 	offs = 0;
 	oldMark = 0;
 	Mpressed = false;
+	drawMark = false;
+	setMouseTracking(true);
+	rulerGesture = new RulerGesture(currView, RulerGesture::VERTICAL);
 	unitChange();
 }
 
 void Vruler::mousePressEvent(QMouseEvent *m)
 {
 	Mpressed = true;
-	if (prefsManager->appPrefs.guidesSettings.guidesShown)
+	if (currDoc->guidesSettings.guidesShown)
 	{
-		QPoint py = currView->viewport()->mapFromGlobal(m->globalPos());
-		currView->DrVX = py.x();
-		qApp->setOverrideCursor(QCursor(SPLITVC), true);
+		qApp->changeOverrideCursor(QCursor(SPLITVC));
+		currView->startGesture(rulerGesture);
 	}
 }
 
 void Vruler::mouseReleaseEvent(QMouseEvent *m)
 {
-	if ((Mpressed) && (m->pos().x() > width()))
+	if (Mpressed)
 	{
-		currView->DrVX = -1;
-		currView->SetXGuide(m, -1);
+		rulerGesture->mouseReleaseEvent(m);
+		Mpressed = false;
 	}
-	qApp->setOverrideCursor(QCursor(ArrowCursor), true);
-	Mpressed = false;
-	currView->updateContents();
 }
 
 void Vruler::mouseMoveEvent(QMouseEvent *m)
 {
-	if ((Mpressed) && (m->pos().x() > width()))
-		currView->FromVRuler(m);
+	if (Mpressed)
+	{
+		rulerGesture->mouseMoveEvent(m);
+	}
+	else
+	{
+		QCursor* cursor = qApp->overrideCursor();
+		Qt::CursorShape shape = cursor ? cursor->shape() : Qt::ArrowCursor;
+		if ((shape == Qt::SplitHCursor) || (shape == Qt::SplitVCursor))
+		{
+			qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
+		}
+	}
 }
 
 void Vruler::paintEvent(QPaintEvent *e)
@@ -94,10 +116,11 @@ void Vruler::paintEvent(QPaintEvent *e)
 	setFont(ff);
 	QPainter p;
 	p.begin(this);
+	p.save();
 	p.setClipRect(e->rect());
 	p.drawLine(16, 0, 16, height());
-	p.setBrush(black);
-	p.setPen(black);
+	p.setBrush(Qt::black);
+	p.setPen(Qt::black);
 	p.setFont(font());
 	double cc = height() / sc;
 	double firstMark = ceil(offs / iter) * iter - offs;
@@ -114,10 +137,10 @@ void Vruler::paintEvent(QPaintEvent *e)
 		int textY = qRound(firstMark * sc)+10;
 		switch (currDoc->unitIndex())
 		{
-			case 1:
+			case SC_MM:
 				tx = QString::number(markC * iter2 / (iter2 / 100) / cor);
 				break;
-			case 2:
+			case SC_IN:
 				xl = (markC * iter2 / iter2) / cor;
 				tx = QString::number(static_cast<int>(xl));
 				frac = fabs(xl - static_cast<int>(xl));
@@ -131,11 +154,11 @@ void Vruler::paintEvent(QPaintEvent *e)
 					tx += QChar(0xBE);
 				tx = tx;
 				break;
-			case 3:
-			case 5:
-				tx = QString::number(markC * iter2 / (iter2 / 10) / cor);
+			case SC_P:
+			case SC_C:
+				tx = QString::number(markC * iter2 / (iter2 /5) / cor);
 				break;
-			case 4:
+			case SC_CM:
 				tx = QString::number(markC * iter2 / iter2 / cor);
 				break;
 			default:
@@ -146,101 +169,81 @@ void Vruler::paintEvent(QPaintEvent *e)
 		firstMark += iter2;
 		markC++;
 	}
+	p.restore();
+	if (drawMark)
+	{
+		QPolygon cr;
+#ifdef OPTION_SMOOTH_MARKERS
+		// draw new marker to pixmap
+		static const int SCALE = 16;
+		static const QColor BACKGROUND(255, 255, 255);
+		static QPixmap pix( 16*SCALE, 4*SCALE );
+		static bool initpix = true;
+		if (initpix)
+		{
+			initpix = false;
+			QPainter pp( &pix );
+			pp.setBrush( BACKGROUND );
+			pp.drawRect( 0, 0, 16*SCALE, 4*SCALE );
+	
+			pp.setPen(Qt::red);
+			pp.setBrush(Qt::red);
+			cr.setPoints(3, 16*SCALE, 2*SCALE, 0, 4*SCALE, 0, 0);
+			pp.drawPolygon(cr);
+		}
+		// draw pixmap
+		p.save();
+		p.translate(0, -currView->contentsY());
+		p.scale(1.0/(SCALE+1), 1.0/SCALE);
+		p.drawPixmap(0, (where-2)*SCALE, pix);
+		p.restore();
+		// repaint marks
+		p.setBrush(Qt::black);
+		p.setPen(Qt::black);
+		p.setFont(font());
+		double sc = currView->getScale();
+		double cc = height() / sc;
+		double firstMark = ceil(offs / iter) * iter - offs;
+		while (firstMark < cc)
+		{
+			p.drawLine(10, qRound(firstMark * sc), 16, qRound(firstMark * sc));
+			firstMark += iter;
+		}
+#else
+		// draw slim marker
+		p.translate(0, -currView->contentsY());
+		p.setPen(Qt::red);
+		p.setBrush(Qt::red);
+		cr.setPoints(5,  5, whereToDraw, 16, whereToDraw, 5, whereToDraw, 0, whereToDraw+2, 0, whereToDraw-2);
+		p.drawPolygon(cr);
+#endif
+	}
 	p.end();
 }
 
 void Vruler::drawNumber(QString num, int starty, QPainter *p)
 {
 	int textY = starty;
-	for (uint a = 0; a < num.length(); ++a)
+	for (int a = 0; a < num.length(); ++a)
 	{
 		QString txt = num.mid(a, 1);
-#ifndef QT_MAC
 		p->drawText(1, textY, txt);
-#else
-		static const int SCALE = 16;
-		QFontMetrics fm = p->fontMetrics();
-		QRect bbox = fm.boundingRect(txt);
-		static QPixmap pix;
-		if (pix.width() < bbox.width()*SCALE || pix.height() < bbox.height()*SCALE)
-			pix.resize(bbox.width()*SCALE, bbox.height()*SCALE);
-		QFont fnt = p->font();
-		QPainter p2;
-		pix.fill();
-		p2.begin( &pix );
-		if (fnt.pointSize() > 0)
-			fnt.setPointSize(SCALE*fnt.pointSize()-SCALE/2);
-		else if (fnt.pixelSize() > 0)
-			fnt.setPixelSize(SCALE*fnt.pixelSize()-SCALE/2);
-		else
-			fnt.setPixelSize(SCALE);
-		p2.setFont(fnt);
-		p2.drawText(-bbox.x()*SCALE, -bbox.y()*SCALE, txt);
-		p2.end();
-		p->scale(1.0/SCALE,1.0/SCALE);
-		p->drawPixmap(1*SCALE, (textY+bbox.top())*SCALE, pix, 0, 0, bbox.width()*SCALE, bbox.height()*SCALE);
-		p->scale(SCALE,SCALE);
-#endif
 		textY += 11;
 	}
 }
 
+double Vruler::ruleSpacing() {
+	return iter;
+}
+
 void Vruler::Draw(int where)
 {
 	// erase old marker
 	int currentCoor = where - currView->contentsY();
+	whereToDraw = where;
+	drawMark = true;
 	repaint(0, oldMark-3, 17, 6);
-	QPointArray cr;
-	QPainter p;
-#ifdef OPTION_SMOOTH_MARKERS
-	// draw new marker to pixmap
-	static const int SCALE = 16;
-	static const QColor BACKGROUND(255, 255, 255);
-	static QPixmap pix( 16*SCALE, 4*SCALE );
-	static bool initpix = true;
-	if (initpix) {
-		initpix = false;
-		p.begin( &pix );
-		p.setBrush( BACKGROUND );
-		p.drawRect( 0, 0, 16*SCALE, 4*SCALE );
-
-		p.setPen(red);
-		p.setBrush(red);
-		cr.setPoints(3, 16*SCALE, 2*SCALE, 0, 4*SCALE, 0, 0);
-		p.drawPolygon(cr);
-		p.end();
-	}
-	// draw pixmap
-	p.begin(this);
-	p.translate(0, -currView->contentsY());
-	p.scale(1.0/(SCALE+1), 1.0/SCALE);
-	p.drawPixmap(0, (where-2)*SCALE, pix);
-	p.end();
-	// restore marks
-	p.begin(this);
-	p.setBrush(black);
-	p.setPen(black);
-	p.setFont(font());
-	double sc = currView->getScale();
-	double cc = height() / sc;
-	double firstMark = ceil(offs / iter) * iter - offs;
-	while (firstMark < cc)
-	{
-		p.drawLine(10, qRound(firstMark * sc), 16, qRound(firstMark * sc));
-		firstMark += iter;
-	}
-	p.end();
-#else
-	// draw slim marker
-	p.begin(this);
-	p.translate(0, -currView->contentsY());
-	p.setPen(red);
-	p.setBrush(red);
-	cr.setPoints(5,  5, where, 16, where, 5, where, 0, where+2, 0, where-2);
-	p.drawPolygon(cr);
-	p.end();
-#endif
-
+//	drawMark = false;
 	oldMark = currentCoor;
 }
 
@@ -251,21 +254,34 @@ void Vruler::unitChange()
 	int docUnitIndex=currDoc->unitIndex();
 	switch (docUnitIndex)
 	{
-		case 0:
+		case SC_PT:
 			if (sc > 1 && sc <= 4)
 				cor = 2;
 			if (sc > 4)
 				cor = 10;
-			iter = unitRulerGetIter1FromIndex(docUnitIndex) / cor;
-	  		iter2 = unitRulerGetIter2FromIndex(docUnitIndex) / cor;
+			if (sc < 0.3)
+			{
+				iter = unitRulerGetIter1FromIndex(docUnitIndex) * 3;
+	  			iter2 = unitRulerGetIter2FromIndex(docUnitIndex) * 3;
+			}
+			else if (sc < 0.2)
+			{
+				iter = unitRulerGetIter1FromIndex(docUnitIndex) * 2;
+	  			iter2 = unitRulerGetIter2FromIndex(docUnitIndex) * 2;
+			}
+			else
+			{
+				iter = unitRulerGetIter1FromIndex(docUnitIndex) / cor;
+	  			iter2 = unitRulerGetIter2FromIndex(docUnitIndex) / cor;
+	  		}
 			break;
-		case 1:
+		case SC_MM:
 			if (sc > 1)
 				cor = 10;
 			iter = unitRulerGetIter1FromIndex(docUnitIndex) / cor;
   			iter2 = unitRulerGetIter2FromIndex(docUnitIndex) / cor;
 			break;
-		case 2:
+		case SC_IN:
 			iter = unitRulerGetIter1FromIndex(docUnitIndex);
 			iter2 = unitRulerGetIter2FromIndex(docUnitIndex);
 			if (sc > 1 && sc <= 4)
@@ -280,11 +296,17 @@ void Vruler::unitChange()
 				iter /= cor;
 				iter2 /= cor;
 			}
+			if (sc < 0.25)
+			{
+				cor = 0.5;
+				iter = 72.0*16.0;
+				iter2 = 72.0*2.0;
+			}
 			break;
-		case 3:
+		case SC_P:
 			iter = unitRulerGetIter1FromIndex(docUnitIndex);
 			iter2 = unitRulerGetIter2FromIndex(docUnitIndex);
-			if (sc > 1 && sc <= 4)
+			if (sc >= 1 && sc <= 4)
 			{
 				cor = 1;
 				iter = 12.0;
@@ -296,19 +318,41 @@ void Vruler::unitChange()
 				iter = 6.0;
 				iter2 = 12.0;
 			}
+			if (sc < 0.3)
+			{
+				cor = 0.25;
+				iter = 12.0*4;
+				iter2 = 60.0*4;
+			}
+			else
+				if (sc < 1)
+			{
+				cor = 1;
+				iter = 12.0;
+				iter2 = 60.0;
+			}
 			break;
-		case 4:
+		case SC_CM:
 			if (sc > 1 && sc <= 4)
 				cor = 1;
 			if (sc > 4)
 				cor = 10;
-			iter = unitRulerGetIter1FromIndex(docUnitIndex) / cor;
-			iter2 = unitRulerGetIter2FromIndex(docUnitIndex) / cor;
+			if (sc < 0.6)
+			{
+				cor=0.1;
+				iter = 720.0/25.4;
+				iter2 = 7200.0/25.4;
+			}
+			else
+			{
+				iter = unitRulerGetIter1FromIndex(docUnitIndex) / cor;
+	  			iter2 = unitRulerGetIter2FromIndex(docUnitIndex) / cor;
+	  		}
 			break;
-		case 5:
+		case SC_C:
 			iter = unitRulerGetIter1FromIndex(docUnitIndex);
 			iter2 = unitRulerGetIter2FromIndex(docUnitIndex);
-			if (sc > 1 && sc <= 4)
+			if (sc >= 1 && sc <= 4)
 			{
 				cor = 1;
 				iter = 72.0/25.4*4.512;
@@ -320,6 +364,19 @@ void Vruler::unitChange()
 				iter = 72.0/25.4*4.512/2.0;
 				iter2 = 72.0/25.4*4.512;
 			}
+			if (sc < 0.3)
+			{
+				cor = 0.1;
+				iter = 72.0/25.4*4.512*10;
+				iter2 = 72.0/25.4*4.512*5.0*10;
+			}
+			else
+				if (sc < 1)
+			{
+				cor = 1;
+				iter = 72.0/25.4*4.512;
+				iter2 = 72.0/25.4*4.512*5.0;
+			}
 			break;
 		default:
 			if (sc > 1 && sc <= 4)
